Camunda external task Robot Framework execution scheduler
=========================================================

**Technology preview.**

`carrot-executor` is an opinionated decoupled Camunda external task executor concept for scheduling Robot Framework RPA tasks. The concept separates task locking from the execution scheduling.

[![](https://mermaid. )](https://mermaid-js.github.io/mermaid-live-editor/edit/## )

In this concept, Carrot external task client, based on [camunda-external-task-client-js](https://github.com/camunda/camunda-external-task-client-js) fetches configured external tasks from Camunda, schedules their execution, and keeps the tasks locked at Camunda until their executor has been completed. The Carrot external task client only completes task (by creating incident) by itself when the scheduling fails with unexpected reason. Any other interaction with Camunda is done by the scheduled Robot Framework bot, mostly using a dedicated Robot Framework listener library.

This initial preview provides support for local parallel task execution, but the concept is designed to support also remote executors, like parameterized Nomad tasks, CI systems, Docker or even Robocloud API.

Requirements:

* Docker with Docker Compose for Camunda

* Python >= 3.8 for executing Robot Framework

  ```bash
  $ python --version
  Python 3.8.8
  ```

* NodeJS >= 12 for executing the external task client

  ```
  $ node --version
  v12.21.0
  ```


Trying it out
=============

While `carrot-executor` itself can be installed from PyPI, trying out the concept requires setting up Camunda BPM Platform and having the example Robot Framework task suites.

The easiest way for all this is to clone or download the project repository and starting the preconfigured Camunda with Docker Compose:

```bash
$ git clone https://github.com/datakurre/carrot-executor
$ cd carrot-executor
$ docker-compose up
```

After everything is ready, there should be Camunda running at http://localhost:8080/camunda with username `demo` and password `demo`.

By default, our Camunda container has both theirs and ours demo processes deployed. Let's get rid of their demo processes:

1. Open Camunda Tasklist: http://localhost:8080/camunda/app/tasklist/default/
2. Choose **Start process**
3. Choose **Reset Camunda to clear state** and
4. Press **Start**

The started process could now be completed with the help of `carrot-executor`. For that we need to create a new Python environment with our package:

```bash
$ python -m venv my-carrot-executor
$ source my-carrot-executor/bin/activate
$ pip install carrot-executor
```

The executor may now be started with parameterizing it to complete tasks from the process we started:

```bash
$ CAMUNDA_API_PATH=http://localhost:8080/engine-rest ROBOT_SUITE=$(pwd)/robot/reset.robot CAMUNDA_TOPIC="Delete all tasklist filters,Delete all deployments" carrot-executor
polling
✓ subscribed to topic Delete all tasklist filters
✓ subscribed to topic Delete all deployments
polling
✓ polled 2 tasks
polling
✓ polled 0 tasks
```

By default, the executor executes the task name matching with the subscribed topic name. This can be overridden with environment variable `ROBOT_TASK`. Setting the variable empty, should execute full suite.
